import { ThemeProvider } from "@material-ui/core/styles";
import { useEffect, useState, useCallback } from "react";
import { Route, Redirect, Switch, useLocation } from "react-router-dom";
import { useDispatch, useSelector } from "react-redux";
import classNames from "classnames";
import { useMediaQuery } from "@material-ui/core";
import { makeStyles } from "@material-ui/core/styles";
import CssBaseline from "@material-ui/core/CssBaseline";
import useTheme from "./hooks/useTheme";
import useBonds from "./hooks/Bonds";
import { useAddress, useWeb3Context } from "./hooks/web3Context";
import { storeQueryParameters } from "./helpers/QueryParameterHelper";
import { shouldTriggerSafetyCheck } from "./helpers";

import { calcBondDetails, getGlobalBondData } from "./slices/BondSlice";
import { loadAppDetails } from "./slices/AppSlice";
import { loadAccountDetails, calculateUserBondDetails, getUserBondData } from "./slices/AccountSlice";
import { error, info } from "./slices/MessagesSlice";

import { Stake, ChooseBond, Bond } from "./views"; // TreasuryDashboard
import Sidebar from "./components/Sidebar/Sidebar.jsx";
import TopBar from "./components/TopBar/TopBar.jsx";
import NavDrawer from "./components/Sidebar/NavDrawer.jsx";
// import LoadingSplash from "./components/Loading/LoadingSplash";
import Messages from "./components/Messages/Messages";
import NotFound from "./views/404/NotFound";

import { dark as darkTheme } from "./themes/dark.js";
import { light as lightTheme } from "./themes/light.js";
import { girth as gTheme } from "./themes/girth.js";
import "./style.scss";
import Wrap from "./views/Wrap/Wrap";
import Calculator from "./views/Calculator/Calculator";
import Exchange from "./views/Exchange/Exchange";
import { RootState } from "./store";
import { Banner } from "./components/banner/banner";
import Mint from "./views/Mint/Mint";
import { FANTOM } from "./helpers/Chains";

const drawerWidth = 300;
const transitionDuration = 969;

const useStyles = makeStyles(theme => ({
  drawer: {
    gridArea: "nav",
    [theme.breakpoints.up(981)]: {
      width: drawerWidth,
      flexShrink: 0,
    },
  },
  content: {
    flexGrow: 1,
    gridArea: "content",
    padding: theme.spacing(1),
    transition: theme.transitions.create("margin", {
      easing: theme.transitions.easing.sharp,
      duration: transitionDuration,
    }),
    height: "100%",
    overflow: "auto",
  },
  contentShift: {
    transition: theme.transitions.create("margin", {
      easing: theme.transitions.easing.easeOut,
      duration: transitionDuration,
    }),
    marginLeft: 0,
  },
  // necessary for content to be below app bar
  toolbar: theme.mixins.toolbar,
  drawerPaper: {
    width: drawerWidth,
  },
}));

function App() {
  // useSegmentAnalytics();
  const dispatch = useDispatch();
  const [theme, toggleTheme, mounted] = useTheme();
  const isAppLoading = useSelector(
    (state: RootState) =>
      state.bonding.loading ||
      state.account.loading ||
      state.app.loading ||
      state.stake.loading
  );
  const location = useLocation();
  const currentPath = location.pathname + location.search + location.hash;
  const classes = useStyles();
  const [isSidebarExpanded, setIsSidebarExpanded] = useState(false);
  const [mobileOpen, setMobileOpen] = useState(false);
  const isSmallerScreen = useMediaQuery("(max-width: 981px)");
  const isSmallScreen = useMediaQuery("(max-width: 600px)");

  const { connect, hasCachedProvider, provider, chainID, connected } = useWeb3Context();
  const address = useAddress();

  const [walletChecked, setWalletChecked] = useState(false);

  let { bonds } = useBonds();

  async function loadDetails(whichDetails: string) {
    // NOTE (unbanksy): If you encounter the following error:
    // Unhandled Rejection (Error): call revert exception (method="balanceOf(address)", errorArgs=null, errorName=null, errorSignature=null, reason=null, code=CALL_EXCEPTION, version=abi/5.4.0)
    // it's because the initial provider loaded always starts with chainID=1. This causes
    // address lookup on the wrong chain which then throws the error. To properly resolve this,
    // we shouldn't be initializing to chainID=1 in web3Context without first listening for the
    // network. To actually test rinkeby, change setChainID equal to 4 before testing.
    let loadProvider = provider;

    if (whichDetails === "app") {
      loadApp(loadProvider);
    }

    // don't run unless provider is a Wallet...
    if (whichDetails === "account" && address && connected && chainID === FANTOM.chainId) {
      loadAccount(loadProvider);
    }
  }

  const loadApp = useCallback(
    async loadProvider => {
      dispatch(loadAppDetails({ networkID: chainID, provider: loadProvider }));
      populateGlobalBondInfo();
    },
    [connected],
  );

  const loadAccount = useCallback(
    async loadProvider => {
      dispatch(loadAccountDetails({ networkID: chainID, address, provider: loadProvider }));
      await dispatch(getUserBondData({ networkID: chainID, provider, address }));
      bonds.map(bond => {
        dispatch(calculateUserBondDetails({ address, bond, provider, networkID: chainID }));
      });
    },
    [connected],
  );

  const populateGlobalBondInfo = useCallback(async () => {
    await dispatch(getGlobalBondData({ networkID: chainID, provider }));
    const newBonds = bonds.filter(bond => !bond.isOld);
    newBonds.map(bond => {
      dispatch(calcBondDetails({ bond, value: "", provider, networkID: chainID }));
    });
  }, []);

  const handleDrawerToggle = () => {
    setMobileOpen(!mobileOpen);
  };

  const handleSidebarClose = () => {
    setIsSidebarExpanded(false);
  };
  let themeMode = theme === "light" ? lightTheme : theme === "dark" ? darkTheme : gTheme;

  // The next 3 useEffects handle initializing API Loads AFTER wallet is checked
  //
  // this useEffect checks Wallet Connection & then sets State for reload...
  // ... we don't try to fire Api Calls on initial load because web3Context is not set yet
  // ... if we don't wait we'll ALWAYS fire API calls via JsonRpc because provider has not
  // ... been reloaded within App.
  useEffect(() => {
    if (hasCachedProvider()) {
      // then user DOES have a wallet
      connect()
        .then(() => {
          setWalletChecked(true);
        })
        .catch(() => {
          setWalletChecked(true);
          dispatch(error("Unlock your Metamask wallet to connect!"));
        });
    } else {
      // then user DOES NOT have a wallet
      setWalletChecked(true);
    }
    // We want to ensure that we are storing the UTM parameters for later, even if the user follows links
    storeQueryParameters();
    if (shouldTriggerSafetyCheck()) {
      // dispatch(info("Safety Check: Always verify you're on app.luxor.money"));
    }
  }, []);

  // this useEffect fires on state change from above. It will ALWAYS fire AFTER
  useEffect(() => {
    // don't load ANY details until wallet is Checked
    if (walletChecked) {
      loadDetails("app");
    }
  }, [walletChecked]);

  // this useEffect picks up any time a user Connects via the button
  useEffect(() => {
    // don't load ANY details until wallet is Connected
    if (connected) {
      loadDetails("account");
    }
  }, [connected]);

  useEffect(() => {
    themeMode = theme === "light" ? lightTheme : darkTheme;
  }, [theme]);

  useEffect(() => {
    if (isSidebarExpanded) handleSidebarClose();
  }, [location]);

  useEffect(() => {
    const updateAppDetailsInterval = setInterval(() => {
      dispatch(loadAppDetails({ networkID: chainID, provider }));
      populateGlobalBondInfo();
    }, 1000 * 120);
    return () => {
      clearInterval(updateAppDetailsInterval);
    };
  }, []);

  useEffect(() => {
    if (walletChecked) {
      const updateAccountDetailInterval = setInterval(async () => {
        dispatch(loadAccountDetails({ networkID: chainID, address, provider: provider }));
        await dispatch(getUserBondData({ networkID: chainID, provider, address }));
        bonds.map(bond => {
          dispatch(calculateUserBondDetails({ address, bond, provider, networkID: chainID }));
        });
      }, 1000 * 30 * 10);
      return () => {
        clearInterval(updateAccountDetailInterval);
      };
    }
  }, [walletChecked, address]);

  return (
    <ThemeProvider theme={themeMode}>
      <CssBaseline />
      <div
        className={classNames("app", theme, {
          tablet: isSmallerScreen && !isSmallScreen,
          mobile: isSmallScreen,
        })}
      >
        <>{false && <Banner isSmallerScreen={isSmallerScreen} isSmallScreen={isSmallScreen}></Banner>}</>
        <Messages />
        {/* header */}
        <TopBar
          theme={theme}
          isAppLoading={isAppLoading}
          toggleTheme={toggleTheme}
          handleDrawerToggle={handleDrawerToggle}
        />
        {/* <nav className={classes.drawer}>
          {isSmallerScreen ? (
            <NavDrawer mobileOpen={mobileOpen} handleDrawerToggle={handleDrawerToggle} />
          ) : (
            <Sidebar />
          )}
        </nav> */}

        <div className={`${classes.content} ${isSmallerScreen && classes.contentShift}`}>
          <Switch>
          <Exchange />

            {/* <Route exact path="/dashboard">
              <TreasuryDashboard />
            </Route> */}

            {/* <Route exact path="/">
              <Redirect to="/stake" />
            </Route>

            <Route path="/stake">
              <Stake />
            </Route>
            <Route path="/wrap">
              <Wrap />
            </Route>
            <Route path="/calculator">
              <Calculator />
            </Route> */}

            {/* <Route path="/bonds">
              {bonds.map(bond => {
                return (
                  <Route exact key={bond.name} path={`/bonds/${bond.name}`}>
                    <>{!bond.isOld && <Bond bond={bond} />}</>
                  </Route>
                );
              })}
              <ChooseBond />
            </Route>
            <Route path="/mint">
              <Mint />
            </Route> */}
              <Route path="/exchange">
                <Exchange />
              </Route>
            <Route component={NotFound} />
          </Switch>
        </div>
      </div>
    </ThemeProvider>
  );
}

export default App;
